/*
 * @Author: Dimitrios Katsaros
 * @Date:   2017-12-06
 */

#ifndef __QTEC_LIB_ARRAY_HIST_H__
#define __QTEC_LIB_ARRAY_HIST_H__

#include <qtec/core/array.h>
#include <glib.h>

G_BEGIN_DECLS

struct _QtecHistInfo {
	gint64 min, max, mode, sum;
	gdouble avg, std_dev;
};
typedef struct _QtecHistInfo QtecHistInfo;

struct _QtecHist {
	guint n_bins;
	guint64 *data;
	QtecHistInfo info;
};
typedef struct _QtecHist QtecHist;

struct _QtecHistArray {
	guint no;
	QtecHist *array;
};
typedef struct _QtecHistArray QtecHistArray;

void qtec_array_free_hist(QtecHist *hist);
void qtec_array_free_hist_array(QtecHistArray *arr);

/* calculate a historgam from QtecArray. The array is handled without
 * taking into account the dimentions of the array.
 * REMEMBER to deallocate the hist after use.
 */
gboolean qtec_array_calc_flattened_histogram(QtecArray *array, QtecHist **hist,
					     guint nr_bins);

/* Similar to above, just handles the last dimention as separate plains.
 * REMEMBER to deallocate the hist array after use.
 */
gboolean qtec_array_calc_image_histogram(QtecArray *array,
					 QtecHistArray **histarr,
					 guint nr_bins);

gdouble qtec_array_hist_get_val_percent(QtecHist *hist, guint bin_nr);

/* Histogram print functions
 * REMEMBER to deallocate the GString* after use with g_string_free()
 */
//Prints the info: min, max, mode, avg, stdev, ...
GString *qtec_array_print_hist_info(QtecHist *hist);
//Prints the bins in the format: [bin_nr]=val (%)
GString *qtec_array_print_hist_bins(QtecHist *hist);
/*Prints an histogram using "ASCII art":
 * [000]=0.000000% | ##
 * [001]=0.000000% | #############
 * [002]=0.176234% | #######
 * [003]=1.210608% | ####################
 * [004]=1.777359% | ###
 * ...
 * Compact: means it wont print empty bins
 * Fit: means it will fit to terminal height (combines bins together)
 */
GString *qtec_array_draw_hist(QtecHist *hist, gboolean compact, gboolean fit);

gboolean qtec_array_calc_equalize_histogram(QtecHist *hist, QtecArray **lut,
					    gboolean sqrt);
gboolean qtec_array_equalize_histogram(QtecArray *array, QtecArray *lut);

G_END_DECLS
#endif
