/*
* @Author: Dimitrios Katsaros
* @Date:   2017-05-04 11:43:45
*/

#ifndef __QTEC_LIB_CONVOLUTION_KERNEL_H__
#define __QTEC_LIB_CONVOLUTION_KERNEL_H__

#include <qtec/v4l2/core.h>

G_BEGIN_DECLS

#define QTEC_CONV_KERNEL_ELEM_SIZE_BYTES 4
#define QTEC_CONV_KERNEL_N_DIMS 2
#define QTEC_CONV_KERNEL_CONVERSION_FACTOR (1 << 14)

typedef enum {
	QTEC_CONV_KERNEL_MONO,
	QTEC_CONV_KERNEL_RED,
	QTEC_CONV_KERNEL_GREEN,
	QTEC_CONV_KERNEL_BLUE,

	QTEC_CONV_KERNEL_LAST,
	QTEC_CONV_KERNEL_INVALID = -1
} QtecConvKernelChannel;

typedef struct _QtecConvolutionKernel QtecConvolutionKernel;

void qtec_conv_kernel_free(QtecConvolutionKernel *krnl);
gboolean qtec_conv_kernel_new(QtecV4l2Dev *dev, QtecConvKernelChannel channel,
			      QtecConvolutionKernel **retkrnl, GError **error);
gboolean qtec_conv_kernel_get(QtecConvolutionKernel *krnl, GError **error);
gboolean qtec_conv_kernel_set(QtecConvolutionKernel *krnl, GError **error);

//returns the kernel_size (square)
guint qtec_conv_kernel_validate_kernel(QtecConvolutionKernel *krnl,
				       GError **error);

gint32 qtec_conv_kernel_get_value(QtecConvolutionKernel *krnl, guint x, guint y,
				  GError **error);
gboolean qtec_conv_kernel_set_value(QtecConvolutionKernel *krnl, guint x,
				    guint y, gint32 val, GError **error);

//test if the value is within the limits defined by the v4l2 control
gboolean qtec_conv_kernel_test_value(QtecConvolutionKernel *krnl, gint32 val,
				     GError **error);

/* The following function create a newely allocated string with the target vaules.
 * Thus, you need to g_free the string after use
 */
gchar *qtec_conv_kernel_to_string(QtecConvolutionKernel *krnl, GError **error);

G_END_DECLS
#endif /* __QTEC_LIB_CONVOLUTION_KERNEL_H__ */
