/*
* @Author: Marianna S. Buschle <msb@qtec.com>
* @Date:   2017-11-02 15:02
*/

#ifndef __QTEC_LIB_CONVOLUTION_OPTIONS_H__
#define __QTEC_LIB_CONVOLUTION_OPTIONS_H__

#include <qtec/convolution/kernel.h>

G_BEGIN_DECLS

typedef enum {
	QTEC_CONV_IDENTITY,
	QTEC_CONV_BOX_BLUR,
	QTEC_CONV_GAUSSIAN_BLUR,
	QTEC_CONV_SHARPEN,
	QTEC_CONV_LAPLACE,
	QTEC_CONV_SOBEL_X,
	QTEC_CONV_SOBEL_Y,
	QTEC_CONV_OUTLINE,
	QTEC_CONV_CUSTOM_KERNEL,
	QTEC_CONV_UNSHARP,
	QTEC_CONV_SHARPEN_OUTLINE,
	QTEC_CONV_SHARPEN_GAUSSIAN,
	QTEC_CONV_UNSHARP_GAUSSIAN,
	QTEC_CONV_LAPLACE_GAUSSIAN,

	QTEC_CONV_OPTIONS_LAST,
	QTEC_CONV_OPTIONS_INVALID = -1
} QtecConvOptions;

enum QtecConvOptionKernelSize {
	QTEC_CONV_KERNEL_SIZE_3X3 = 3,
	QTEC_CONV_KERNEL_SIZE_5X5 = 5
};

typedef struct _QtecConvOptionInfo QtecConvOptionInfo;
typedef struct _QtecConvOptionKernel QtecConvOptionKernel;

QtecConvOptionKernel *qtec_conv_options_new_custom_kernel(guint size);
void qtec_conv_options_free_custom_kernel(QtecConvOptionKernel **kernel_data);

gboolean qtec_conv_options_get_kernel(QtecConvOptions type,
				      QtecConvOptionKernel *kernel_data,
				      GError **error);
gboolean qtec_conv_options_get_info(QtecConvOptions type,
				    const QtecConvOptionInfo **info,
				    GError **error);
gboolean qtec_conv_options_get_nth_info(gint index,
					const QtecConvOptionInfo **info,
					GError **error);
gint qtec_conv_options_get_info_no(void);

guint qtec_conv_options_get_kernel_size(QtecConvOptionKernel *kernel_data);
void qtec_conv_options_set_kernel_size(QtecConvOptionKernel *kernel_data,
				       guint size);

gint qtec_conv_options_get_type(const QtecConvOptionInfo *info);
const gchar *qtec_conv_options_get_name(const QtecConvOptionInfo *info);
const gchar *qtec_conv_options_get_description(const QtecConvOptionInfo *info);

gboolean qtec_conv_options_apply_kernel(QtecV4l2Dev *dev,
					QtecConvKernelChannel channel,
					QtecConvOptions type, gdouble gain,
					gboolean normalize, GError **error);

/* Sets the values from the gdouble* contained in kernel_data
 * into the QtecConvolutionKernel,
 * optionally applying a gain to all coefficients
 */
gboolean qtec_conv_options_set_array(QtecConvolutionKernel *krnl,
				     QtecConvOptionKernel *kernel_data,
				     gdouble gain, gboolean normalize,
				     GError **error);

// Calculates the normalization factor for the kernel
gboolean qtec_conv_options_calc_norm(QtecConvOptionKernel *kernel_data,
				     gdouble *norm, GError **error);

/* Calculates the minimum and maximum allowed gains so that the kernel values
 * don't go over the control min and max
 */
gboolean qtec_conv_options_calc_min_max_gain(QtecConvolutionKernel *krnl,
					     QtecConvOptionKernel *kernel_data,
					     gdouble *min_gain,
					     gdouble *max_gain, GError **error);

gboolean qtec_conv_options_get_kernel_value(QtecConvOptionKernel *kernel_data,
					    guint x, guint y, gdouble *value,
					    GError **error);
gboolean qtec_conv_options_set_kernel_value(QtecConvOptionKernel *kernel_data,
					    guint x, guint y, gdouble value,
					    GError **error);

G_END_DECLS
#endif /* __QTEC_LIB_CONVOLUTION_KERNEL_OPTIONS_H__ */
