/*
 * @Author: Dimitrios Katsaros
 * @Date:   2018-01-16
 */

/* set of utility functions for interaction with a file descriptor */

#ifndef __QTEC_LIB_CORE_FD_H__
#define __QTEC_LIB_CORE_FD_H__

#include <glib.h>

G_BEGIN_DECLS

/* normal ioctl with slightly different interface and with handling for
 * EINTR cases
 */
gboolean qtec_core_fd_ioctl(gint fh, gulong request, gpointer arg);
/* Like above but with an extra field for the return value.
 * Rarely used, but useful for some ioctls
 */
gboolean qtec_core_fd_ioctl_return(gint fh, gulong request, gpointer arg,
				   gint *retval);

/* Like above but returns the errno string in the GError.
 * Useful fx for the EBUSY case when changing image size while STREAM_ON
 */
gboolean qtec_core_fd_ioctl_ret_err(gint fh, gulong request, gpointer arg,
				    GError **error);

/* same as close but with different interface */
gboolean qtec_core_fd_close(int fd);
/* Open useful for opening fds. The device is opened in non-blocking RW mode
 * Also, checks are performed before opening the device, like if the location
 * specified is valid and that the target is actually a device. All failure
 * cases stated or if the open fails cases the function to return false.
 */
gboolean qtec_core_fd_open(const gchar *device_file, gint *retfd);

G_END_DECLS

#endif
