/*
 * @Author: Dimitrios Katsaros
 * @Date:   2017-12-06
 */

/*
 * A very simple iterator implementation for handling items that need
 * to be iterated over. Loosely based on GstIterator from GStreamer
 */

#ifndef __QTEC_LIB_CORE_ITERATOR_H__
#define __QTEC_LIB_CORE_ITERATOR_H__

#include <glib-object.h>

G_BEGIN_DECLS

#define QTEC_ITERATOR_CAST(itr) ((QtecIterator *)itr)

typedef struct _QtecIterator QtecIterator;

typedef enum {
	QTEC_ITERATOR_ERROR,
	QTEC_ITERATOR_COMPLETE,
	QTEC_ITERATOR_ITEM,
} QtecIteratorState;

typedef QtecIteratorState (*QtecIteratorNextFunc)(QtecIterator *itor,
						  GValue *item);
typedef void (*QtecIteratorFreeFunc)(QtecIterator *itor);

struct _QtecIterator {
	gsize size;
	GType type;
	QtecIteratorNextFunc next;
	QtecIteratorFreeFunc free;
};

/* Core constructor for all iterators
 * iteratorsize: The size of the iterator structure
 * itemtype: The gtype of the returned items
 * next: The function called when qtec_iterator_next is called
 * free: The function called when the iterator is freed
 */
QtecIterator *qtec_iterator_new(gsize iteratorsize, GType itemtype,
				QtecIteratorNextFunc next,
				QtecIteratorFreeFunc free);

/* Get the next item from the iterator */
QtecIteratorState qtec_iterator_next(QtecIterator *itor, GValue *item);

/* Frees the iterator and all asociated resources */
void qtec_iterator_free(QtecIterator *itor);

G_END_DECLS

#endif
