/*
* @Author: Dimitrios Katsaros
* @Date:   2017-05-04 11:43:45
*/

#ifndef __QTEC_LIB_CORE_MACROS_H__
#define __QTEC_LIB_CORE_MACROS_H__

#define QTEC_ZERO_INIT \
	{              \
	}
#define QTEC_BOOLIFY(val) (!!(val))
#define QTEC_BIT(p) (1ULL << p)
#define QTEC_GET_BIT_FROM_VAL(v, p) (QTEC_BOOLIFY(v & QTEC_BIT(p)))

#define QTEC_DOUBLE_PTR_CHECK(dbl_ptr) (dbl_ptr && (*dbl_ptr == NULL))

/* I am deliberately not including <qtec/core/error.h> here
 * The reason is that we do not need to pollute the macro header with
 * includes when including the header it's self does not break anything.
 * If you find your self needing a header for a macro just include the header
 * on macro use
 */

#define qtec_return_gerror_if_fail(expr, error_ptr)                       \
	G_STMT_START                                                      \
	{                                                                 \
		if G_LIKELY (expr) {                                      \
		} else {                                                  \
			g_set_error(error_ptr, QTEC_CORE_ERROR,           \
				    QTEC_CORE_ERROR_PARAM_NOT_SET,        \
				    "Pointer parameters not set");        \
			g_return_if_fail_warning(G_LOG_DOMAIN, G_STRFUNC, \
						 #expr);                  \
			return;                                           \
		};                                                        \
	}                                                                 \
	G_STMT_END

#define qtec_return_gerror_and_val_if_fail(expr, val, error_ptr)          \
	G_STMT_START                                                      \
	{                                                                 \
		if G_LIKELY (expr) {                                      \
		} else {                                                  \
			g_set_error(error_ptr, QTEC_CORE_ERROR,           \
				    QTEC_CORE_ERROR_PARAM_NOT_SET,        \
				    "Pointer parameters not set");        \
			g_return_if_fail_warning(G_LOG_DOMAIN, G_STRFUNC, \
						 #expr);                  \
			return val;                                       \
		};                                                        \
	}                                                                 \
	G_STMT_END

#endif
