/*
* @Author: Marianna S. Buschle <msb@qtec.com>
* @Date:   2017-11-8 15:42
*/

#ifndef __QTEC_LIB_CORE_MATH_H__
#define __QTEC_LIB_CORE_MATH_H__

#include <glib.h>

G_BEGIN_DECLS
#ifndef QTEC_MATH_PI
#define QTEC_MATH_PI 3.14159265358979323846
#endif

gint32 qtec_math_double_to_raw(gdouble value, guint conversion_factor);
gdouble qtec_math_raw_to_double(gint32 value_raw, guint conversion_factor);

gdouble qtec_math_log_base_x(guint base, gdouble x);

/* Fraction approximation, maxden sets the detail of the approximation by setting
 * the maximum value that the denominator of the fraction can have
 */
void qtec_math_approx_double(gdouble real, gint maxden, gint *num, gint *denom);

gint qtec_math_abs(gint x);
gdouble qtec_math_pow(gdouble x, gdouble y);
gdouble qtec_math_round(gdouble x);
gdouble qtec_math_sqrt(gdouble x);
gdouble qtec_math_modf(gdouble x, gdouble *iptr);
gdouble qtec_math_sin(gdouble x);
gdouble qtec_math_cos(gdouble x);
gdouble qtec_math_tan(gdouble x);
gdouble qtec_math_fabs(gdouble x);
gdouble qtec_math_sin_deg(gdouble x);
gdouble qtec_math_cos_deg(gdouble x);
gdouble qtec_math_tan_deg(gdouble x);
gdouble qtec_math_atan2(gdouble y, gdouble x);
gdouble qtec_math_atan2_deg(gdouble y, gdouble x);

static inline gdouble qtec_math_to_radians(gdouble degrees)
{
	return degrees * (QTEC_MATH_PI / 180.0);
}

static inline gdouble qtec_math_to_degrees(gdouble radians)
{
	return radians * (180.0 / QTEC_MATH_PI);
}

gdouble qtec_math_ceil(gdouble x);
gdouble qtec_math_floor(gdouble x);

gdouble qtec_math_interpolate(gdouble x1, gdouble y1, gdouble x2, gdouble y2,
			      gdouble x);

G_END_DECLS
#endif
