/*
 * @Author: Rohollah Javaheri <rohollah@qtec.com>
 * @Date:   2019-09-24
 */

#ifndef __QTEC_LIB_CORE_MATRIX_H__
#define __QTEC_LIB_CORE_MATRIX_H__

#include <glib.h>

G_BEGIN_DECLS

struct QtecMatrix {
	gdouble *data;
	guint rows;
	guint cols;
};

//matrix_ptr must be a 'struct QtecMatrix*'
#define QTEC_CORE_MATRIX_AT(matrix_ptr, row, col) \
	(matrix_ptr->data[row * matrix_ptr->cols + col])

struct QtecMatrix *qtec_core_matrix_new(guint rows, guint cols);
void qtec_core_matrix_free(struct QtecMatrix *m);

/* This function perform the cross_product of two matrixes (x1 and x2)
 *
 * @x1 is the first matrix (left-side of the operation)
 * @x2 is the second matrix (right-side of the operation)
 * @res is the output of the operation (res = x1 * x2)
 * which is newly allocated as must be freed when done
 * with qtec_core_matrix_free()
 *
 * Also note that the matrix dimension rules must be respected:
 * [M, N] * [N, O] = [M, O]
 */
gboolean qtec_core_matrix_cross_product(struct QtecMatrix *x1,
					struct QtecMatrix *x2,
					struct QtecMatrix **res);
/* Transposes matrix m
 * returning the result in the newly allocated matrix res
 * which must be freed when done with qtec_core_matrix_free()
 *
 * if m is [M, N], then res will be [N, M]
 */
gboolean qtec_core_matrix_transpose(struct QtecMatrix *m,
				    struct QtecMatrix **res);

//The returned GString* must be deallocated after use with g_string_free()
GString *qtec_core_matrix_print(struct QtecMatrix *m);

//copy values from a gdouble[][]
gboolean qtec_core_matrix_copy_vals(struct QtecMatrix *m, gdouble *data);

G_END_DECLS
#endif
