/*
 * @Author: Dimitrios Katsaros
 * @Date:   2017-11-16
 */

/*
 * A set of basic parsing functions
 */

#ifndef __QTEC_LIB_CORE_PARSE_H__
#define __QTEC_LIB_CORE_PARSE_H__

#include <glib.h>

/* *INDENT-OFF* */
G_BEGIN_DECLS
/* *INDENT-ON* */
typedef gboolean (*QtecCoreParseCharFunc)(gchar c);

/* wrapper function around g_ascii_isspace, since g_ascii_isspace is a macro
 * and cannot be passed as a function pointer
 */
gboolean qtec_core_parse_is_space(gchar c);

/* parses a single digit 0-9 */
gboolean qtec_core_parse_digit(const gchar *string, gchar **retendpos,
			       gint *value, GError **error);
gboolean qtec_core_parse_int(const gchar *string, gchar **retendpos,
			     gint *value, GError **error);
gboolean qtec_core_parse_uint(const gchar *string, gchar **retendpos,
			      guint *value, GError **error);
gboolean qtec_core_parse_int64(const gchar *string, gchar **retendpos,
			       gint64 *value, GError **error);
gboolean qtec_core_parse_double(const gchar *string, gchar **retendpos,
				gdouble *value, GError **error);
gboolean qtec_core_parse_bitmask(const gchar *string, gchar **retendpos,
				 guint64 *value, GError **error);
gboolean qtec_core_parse_boolean(const gchar *str, gchar **retendpos,
				 gboolean *res, GError **error);

/* parses a single array position in the format [<pos>]:<val>
 */
gboolean qtec_core_parse_int_array_single_position(const gchar *str,
						   gchar **retendpos,
						   guint *position, gint *value,
						   GError **error);
/* Parses a single array value that gets applied to the entire
 * array, in the format [:]:<val>
 */
gboolean qtec_core_parse_int_array_wildcard(const gchar *str, gchar **retendpos,
					    gint *value, GError **error);
/* Parses an entire array. The format is []:<val1>,<val2>,...<valn>
 * This expects the input size to be equal to the array size
 */
gboolean qtec_core_parse_int_array_all_positions(const gchar *str,
						 gchar **retendpos, gint arr[],
						 guint array_size,
						 GError **error);
/* performs all the previous array parsing functions
 */
gboolean qtec_core_parse_int_array(const gchar *str, gchar **retendpos,
				   gint arr[], guint array_size,
				   GError **error);
gboolean qtec_core_parse_eat_char(const gchar *str, gchar **retendpos, gchar c,
				  GError **error);
/* Like the above function but uses a function for the check. Useful
 * for more elaborate checks. retc can be set to a gchar to get the char that
 * was consumed successfully.
 */
gboolean qtec_core_parse_eat_char_using_func(const gchar *str,
					     gchar **retendpos,
					     QtecCoreParseCharFunc func,
					     gchar *retc, GError **error);
gboolean qtec_core_parse_eat_whitespace(const gchar *str, gchar **retendpos,
					GError **error);
gboolean qtec_core_parse_eat_whitespaces(const gchar *str, gchar **retendpos,
					 GError **error);
gboolean qtec_core_parse_is_nul(const gchar *str, GError **error);
G_END_DECLS
#endif
