/*
 * @Author: Marianna S Buschle
 * @Date:   2018-06-25
 */

#ifndef __QTEC_LIB_GEOMETRY_CORE_H__
#define __QTEC_LIB_GEOMETRY_CORE_H__

#include <glib.h>

G_BEGIN_DECLS

typedef struct {
	gdouble x;
	gdouble y;
} QtecGeometryPoint2d;

typedef struct {
	gdouble x;
	gdouble y;
	gdouble z;
} QtecGeometryPoint3d;

typedef struct {
	QtecGeometryPoint2d p1;
	QtecGeometryPoint2d p2;
	QtecGeometryPoint2d p3;
	QtecGeometryPoint2d p4;
} QtecGeometryQuad2d;

//Euclidean distance between 2 points
gdouble qtec_geometry_calc_point_dist(QtecGeometryPoint2d p0,
				      QtecGeometryPoint2d p1);

//Test point equality
gboolean qtec_geometry_points_equal(QtecGeometryPoint2d p0,
				    QtecGeometryPoint2d p1);

//Test if point A, B and C all lie in the same line
//fx if AB is the longest segment:
//distance(A, C) + distance(B, C) == distance(A, B)
//'precision' specifies the error allowed, must be > 0
//in order to account for double "precision"
//negative values will cause the default precision = 0.01 to be used
//'debug' allows for printing debug information in case the result is FALSE
gboolean qtec_geometry_test_colinear_points(QtecGeometryPoint2d a,
					    QtecGeometryPoint2d b,
					    QtecGeometryPoint2d c,
					    gdouble precision, gboolean debug);

//Quadrilateral equality
gboolean qtec_geometry_quads_equal(QtecGeometryQuad2d *q0,
				   QtecGeometryQuad2d *q1);

//Bilinear interpolation
//Returns the value at point p given the 4 surrounding values
//and their x and y coordinates
gdouble qtec_geometry_bilinear_interpolation(QtecGeometryPoint2d p, gdouble x1,
					     gdouble x2, gdouble y1, gdouble y2,
					     gdouble val11, gdouble val12,
					     gdouble val21, gdouble val22);

gdouble qtec_geometry_euclidean_distance(QtecGeometryPoint3d p1,
					 QtecGeometryPoint3d p2);

G_END_DECLS
#endif
