/* GStreamer
 *
 * Copyright (C) 2017 Dimitrios Katsaros <patcherwork@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef __QTEC_LIB_GST_PIPE_HANDLER_H__
#define __QTEC_LIB_GST_PIPE_HANDLER_H__

/*
 * A Generic threaded pipeline handler for simplifying application
 * development. provides simple functions for playing and stoping the
 * pipe, signals for messages that can be filtered and all of this
 * in an interface free of GStreamer dependencies. Based on the GstPlayer
 * in bad:
 * https://cgit.freedesktop.org/gstreamer/gst-plugins-bad/tree/gst-libs/gst/player
 */

#include <gst/gst.h>

G_BEGIN_DECLS

typedef struct _QtecGstPipeHandler QtecGstPipeHandler;
typedef struct _QtecGstPipeHandlerClass QtecGstPipeHandlerClass;

#define QTEC_GST_PIPE_TYPE_HANDLER (qtec_gst_pipe_handler_get_type())
#define QTEC_GST_PIPE_HANDLER(obj) ((QtecGstPipeHandler *)(obj))

GType qtec_gst_pipe_handler_get_type(void);
QtecGstPipeHandler *qtec_gst_pipe_handler_new(GstElement *pipe);
void qtec_gst_pipe_handler_unref(QtecGstPipeHandler *hdlr);
QtecGstPipeHandler *qtec_gst_pipe_handler_ref(QtecGstPipeHandler *hdlr);

void qtec_gst_pipe_handler_add_window_id(QtecGstPipeHandler *hdlr,
					 gpointer eleptr, guintptr wid);
void qtec_gst_pipe_handler_set_playing(QtecGstPipeHandler *hdlr);
void qtec_gst_pipe_handler_set_paused(QtecGstPipeHandler *hdlr);
void qtec_gst_pipe_handler_stop(QtecGstPipeHandler *hdlr);
GstElement *qtec_gst_pipe_handler_lookup_element(QtecGstPipeHandler *self,
						 const gchar *name);
void qtec_gst_pipe_handler_call_within_context(QtecGstPipeHandler *hdlr,
					       GSourceFunc func,
					       gpointer user_data,
					       GDestroyNotify notify);
void qtec_gst_pipe_handler_seek(QtecGstPipeHandler *self, gint64 position);

G_END_DECLS
#endif
