/*
* @Author: Dimitrios Katsaros
* @Date:   2017-05-04 11:43:45
*/

/* translation layer between gst types and v4l2 types */

#ifndef __QTEC_LIB_GST_V4L2_H__
#define __QTEC_LIB_GST_V4L2_H__

#include <gst/gst.h>
#include <gst/video/video.h>
#include <qtec/v4l2/core.h>

G_BEGIN_DECLS

guint32 qtec_gst_v4l2_video_format_to_fourcc(GstVideoFormat gstfmt,
					     guint32 *alternative);
guint32 qtec_gst_v4l2_str_format_to_fourcc(const gchar *format,
					   guint32 *alternative);

GstVideoFormat qtec_gst_v4l2_fourcc_to_video_format(guint32 fourcc);

void qtec_gst_v4l2_caps_to_gvalue(struct v4l2_capability *caps, GValue *target);
void qtec_gst_v4l2_query_to_gvalue(struct v4l2_query_ext_ctrl *qctrl,
				   GValue *target);
void qtec_gst_v4l2_flags_to_gvalue(struct v4l2_query_ext_ctrl *qctrl,
				   GValue *target);
void qtec_gst_v4l2_selection_to_gvalue(struct v4l2_selection *sel,
				       GValue *target);
void qtec_gst_v4l2_menu_to_gvalue(struct v4l2_querymenu *qmenu, guint type,
				  GValue *target);
/* this one is a bit different, since we need the device wrapper to correctly handle
 * the conversion of the value to a gvalue. Since we are passing the wrapper, we make our life
 * easier also by not requiring the user to pass the query that corresponds to the control.
 */
gboolean qtec_gst_v4l2_ext_ctrl_to_g_value(QtecV4l2Dev *dev,
					   struct v4l2_ext_control *ctrl,
					   GValue *target);

/* Returns an allocated list of supported formats */
gboolean qtec_gst_v4l2_get_formats(QtecV4l2Dev *dev, guint32 type,
				   GValue *retfmts);
/* the set gvalue may be of type intrange or list depending on returned sizes from the device */
gboolean qtec_gst_v4l2_get_frame_sizes(QtecV4l2Dev *dev, guint32 pixelformat,
				       GValue *retfrs);
/* the returned gvalue can be of type GST_TYPE_FRACTION_RANGE or GST_TYPE_LIST */
gboolean qtec_gst_v4l2_get_frame_intervals(QtecV4l2Dev *dev,
					   guint32 pixelformat, guint32 width,
					   guint32 height, GValue *retfis);

/* iterators */
GstIterator *qtec_gst_v4l2_query_iterator_new(QtecV4l2Dev *dev, GMutex *lock,
					      guint32 *master_cookie,
					      gboolean subscribe_events,
					      gboolean allow_event_feedback);
GstIterator *qtec_gst_v4l2_query_menu_iterator_new(QtecV4l2Dev *dev,
						   GMutex *lock,
						   guint32 *master_cookie,
						   guint ctrlid);
GstIterator *qtec_gst_v4l2_get_ext_ctrl_iterator_new(QtecV4l2Dev *dev,
						     GMutex *lock,
						     guint32 *master_cookie,
						     guint ctrlclassid);
G_END_DECLS
#endif
