/*
 * @Author: Dimitrios Katsaros
 * @Date:   2017-11-30
 */

#ifndef __QTEC_IO_QTEC_ARRAY__
#define __QTEC_IO_QTEC_ARRAY__

/* a simple interface for reading/writing qtec arrays to files. The
 * difference from standardized formats like PAM is that there are no
 * limitations on the size and complexity of the inidividual members. Thus
 * with this you can save structures of any size. To identify the type
 * of structure written to the file, an identifier is passed. Do note that
 * this format is not reprisentable. If you need to save an array of simple
 * types of size 16bits or smaller, use PAM instead.
 */

#include <gio/gio.h>
#include <qtec/core/array.h>

G_BEGIN_DECLS

gboolean qtec_io_array_read_from_stream(GInputStream *stream,
					gchar **identifier, gsize ele_size,
					QtecArray **array,
					GCancellable *cancellable,
					GError **error);
gboolean qtec_io_array_write_to_stream(GOutputStream *ostrm,
				       const gchar *identifier,
				       QtecArray *array,
				       GCancellable *cancellable,
				       GError **error);
gboolean qtec_io_array_read_from_file(const gchar *fileloc, gchar **ident,
				      gsize ele_size, QtecArray **array,
				      GCancellable *cancellable,
				      GError **error);
gboolean qtec_io_array_write_to_file(const gchar *fileloc,
				     const gchar *identifier, QtecArray *array,
				     GCancellable *cancellable, GError **error);

G_END_DECLS
#endif
