/*
* @Author: Marianna S. Buschle <msb@qtec.com>
* @Date:   2017-11-8 15:42
*/

#ifndef __QTEC_LIB_LUT_OPTIONS_H__
#define __QTEC_LIB_LUT_OPTIONS_H__

#include <qtec/lut/core.h>
#include <qtec/core/array.h>

G_BEGIN_DECLS

typedef enum {
	QTEC_LUT_OPTIONS_THRESHOLD,
	QTEC_LUT_OPTIONS_RANGE,
	QTEC_LUT_OPTIONS_INVERT,
	QTEC_LUT_OPTIONS_GAMMA,
	QTEC_LUT_OPTIONS_QUANTIZATION,
	QTEC_LUT_OPTIONS_INTENSITY_STRETCH,
	QTEC_LUT_OPTIONS_CUSTOM,
	QTEC_LUT_OPTIONS_LINEAR,
	QTEC_LUT_OPTIONS_PIECEWISE_LINEAR,
	QTEC_LUT_OPTIONS_CONTRAST_MASK,
	QTEC_LUT_OPTIONS_S_CURVE,
	QTEC_LUT_OPTIONS_HISTOGRAM_EQUALIZE,

	QTEC_LUT_OPTIONS_LAST,
	QTEC_LUT_OPTIONS_INVALID = -1
} QtecLutOptions;

typedef enum {
	QTEC_LUT_THRESHOLD_BINARY,
	QTEC_LUT_THRESHOLD_TRUNCATE,
	QTEC_LUT_THRESHOLD_TO_ZERO,
	QTEC_LUT_THRESHOLD_TO_ZERO_INVERTED,

	QTEC_LUT_THRESHOLD_LAST,
	QTEC_LUT_THRESHOLD_INVALID = -1
} QtecLutThreshold;

typedef struct _QtecLutOptionsInfo QtecLutOptionsInfo;
typedef struct _QtecLutThresholdInfo QtecLutThresholdInfo;

gboolean qtec_lut_options_get_info(QtecLutOptions type,
				   const QtecLutOptionsInfo **info,
				   GError **error);
gboolean qtec_lut_options_get_nth_info(gint index,
				       const QtecLutOptionsInfo **info,
				       GError **error);
gint qtec_lut_options_get_info_no(void);

QtecLutOptions qtec_lut_options_get_type(const QtecLutOptionsInfo *info);
const gchar *qtec_lut_options_get_info_name(const QtecLutOptionsInfo *info);
const gchar *
qtec_lut_options_get_info_description(const QtecLutOptionsInfo *info);

gboolean qtec_lut_options_get_thres_info(QtecLutThreshold type,
					 const QtecLutThresholdInfo **info,
					 GError **error);
gboolean qtec_lut_options_get_nth_thres_info(gint index,
					     const QtecLutThresholdInfo **info,
					     GError **error);
gint qtec_lut_options_get_thres_info_no(void);

QtecLutThreshold
qtec_lut_options_get_thres_type(const QtecLutThresholdInfo *info);
const gchar *qtec_lut_options_get_thres_name(const QtecLutThresholdInfo *info);
const gchar *
qtec_lut_options_get_thres_description(const QtecLutThresholdInfo *info);

gboolean qtec_lut_options_get_name(QtecLutOptions type, const gchar **name,
				   GError **error);
gboolean qtec_lut_options_get_description(QtecLutOptions type,
					  const gchar **description,
					  GError **error);

gboolean qtec_lut_options_threshold_get_name(QtecLutThreshold type,
					     const gchar **name,
					     GError **error);
gboolean qtec_lut_options_threshold_get_description(QtecLutThreshold type,
						    const gchar **description,
						    GError **error);

void qtec_lut_options_calc_identity(QtecLutTable *lut);
gboolean qtec_lut_options_calc_threshold(QtecLutTable *lut, gint value,
					 QtecLutThreshold type, GError **error);
gboolean qtec_lut_options_calc_range(QtecLutTable *lut, gint value_low,
				     gint value_high, GError **error);
void qtec_lut_options_calc_invert(QtecLutTable *lut);
gboolean qtec_lut_options_calc_gamma(QtecLutTable *lut, gdouble gamma,
				     GError **error);
gboolean qtec_lut_options_calc_quantization(QtecLutTable *lut, gdouble nr_steps,
					    GError **error);
gboolean qtec_lut_options_calc_intensity_stretch(QtecLutTable *lut,
						 gint min_intensity,
						 gint max_intensity,
						 GError **error);
gboolean qtec_lut_options_calc_contrast_mask(QtecLutTable *lut, GError **error);
gboolean qtec_lut_options_calc_Scurve(QtecLutTable *lut, gint mid_point,
				      gdouble force, GError **error);
gboolean qtec_lut_options_calc_linear(QtecLutTable *lut, gdouble a, gdouble b,
				      GError **error);
gboolean qtec_lut_options_calc_piecewise_linear(QtecLutTable *lut, gint x1,
						gint y1, gint x2, gint y2,
						GError **error);
gboolean qtec_lut_options_calc_histogram_equalize(QtecLutTable *lut,
						  QtecArray *array,
						  gboolean use_sqrt,
						  GError **error);

G_END_DECLS
#endif
