/*
 * @Author: Rohollah Javaheri
 * @Date:   2019-06-28
 */

#ifndef __QTEC_LIB_POLARIZED_CAM_H__
#define __QTEC_LIB_POLARIZED_CAM_H__

#include <qtec/array/color.h>
#include <qtec/array/getset.h>
#include <qtec/array/math.h>
#include <qtec/core/array.h>
#include <qtec/core/debug.h>
#include <qtec/core/math.h>
#include <qtec/io/io.h>
#include <glib.h>

G_BEGIN_DECLS

typedef struct _QtecPolarizerImages QtecPolarizerImages;

// Return the polarization image at direction of 0 degree
QtecArray *qtec_polarizedcam_get_dir_0(QtecPolarizerImages *pimages);

// Return the polarization image at direction of 45 degree
QtecArray *qtec_polarizedcam_get_dir_45(QtecPolarizerImages *pimages);

// Return the polarization image at direction of 90 degree
QtecArray *qtec_polarizedcam_get_dir_90(QtecPolarizerImages *pimages);

// Return the polarization image at direction of 135 degree
QtecArray *qtec_polarizedcam_get_dir_135(QtecPolarizerImages *pimages);

// Return the stoke vector image of S0
QtecArray *qtec_polarizedcam_get_s0(QtecPolarizerImages *pimages);

// Return the stoke vector image of S1
QtecArray *qtec_polarizedcam_get_s1(QtecPolarizerImages *pimages);

// Return the stoke vector image of S2
QtecArray *qtec_polarizedcam_get_s2(QtecPolarizerImages *pimages);

// Return Imax image
QtecArray *qtec_polarizedcam_get_imax(QtecPolarizerImages *pimages);

// Return Imin image
QtecArray *qtec_polarizedcam_get_imin(QtecPolarizerImages *pimages);

// Return the image of "Degree of polarization"
QtecArray *qtec_polarizedcam_get_dolp(QtecPolarizerImages *pimages);

// Return the image of "Angle of polarization"
QtecArray *qtec_polarizedcam_get_aop(QtecPolarizerImages *pimages);

/* Return the 3-channels HSV image. The HSV image is build as follow:
 *  - Hue:	  Angle of polarization
 *  - Saturation: Degree of polarization
 *  - Value:	  Stoke vector S0
 */
QtecArray *qtec_polarizedcam_get_hsv(QtecPolarizerImages *pimages);

// Release all the allocated memories
gboolean qtec_polarizedcam_free(QtecPolarizerImages *pimages);

// Allocate memory
gboolean qtec_polarizedcam_new(QtecArray *arr, QtecPolarizerImages **pimages);

// Return the directions and polarized images
gboolean qtec_polarizedcam_get_images(QtecArray *arr,
				      QtecPolarizerImages *pimages);

G_END_DECLS
#endif
