/*
 * @Author: Dimitrios Katsaros
 * @Date:   2017-12-06
 */

#ifndef __QTEC_LIB_XFORM_CORE_H__
#define __QTEC_LIB_XFORM_CORE_H__

#include <glib-object.h>
#include <qtec/core/array.h>
#include <qtec/xform/gain/map.h>
#include <qtec/xform/distortion/map.h>

#include <gio/gio.h>

G_BEGIN_DECLS

/* forward declaring v4l2 dev to not need an include */
struct qtec_distortion;
typedef struct _QtecV4l2Dev QtecV4l2Dev;
typedef struct _QtecXForm QtecXForm;

void qtec_xform_free_device(QtecXForm *xform);
gboolean qtec_xform_open_from_v4l2dev(QtecV4l2Dev *v4l2dev, QtecXForm **xform);

/* these reflect the v4l2 device controls */
gboolean qtec_xform_set_gain(QtecXForm *xform, gboolean active);
gboolean qtec_xform_set_dist(QtecXForm *xform, gboolean active);

gboolean qtec_xform_write_gain(QtecXForm *xform, QtecXFormGainMap *map);
gboolean qtec_xform_write_dist(QtecXForm *xform, QtecXFormDistMap *map);
gboolean qtec_xform_read_gain(QtecXForm *xform, QtecXFormGainMap **retmap);
gboolean qtec_xform_read_dist(QtecXForm *xform, QtecXFormDistMap **retmap);

typedef enum {
	QTEC_XFORM_ZERO_INIT,
	QTEC_XFORM_IDENTITY,
} QtecXFormBuildType;

/* functions for building defaults of xform arrays */
gboolean qtec_xform_build_gain_map(QtecXForm *xform, QtecXFormBuildType btype,
				   QtecXFormGainMap **retmap);
/* unlike the gain, the dist only makes sense as an "identity" */
gboolean qtec_xform_build_dist_map(QtecXForm *xform, QtecXFormDistMap **retmap);

/* file operations */
gboolean qtec_xform_read_gain_from_file(const gchar *fileloc,
					QtecXFormGainMap **retmap,
					GError **error);
gboolean qtec_xform_write_gain_to_file(const gchar *fileloc,
				       QtecXFormGainMap *map, GError **error);
gboolean qtec_xform_write_gain_to_stream(GOutputStream *stream,
					 QtecXFormGainMap *map, GError **error);
gboolean qtec_xform_read_dist_from_file(const gchar *fileloc,
					QtecXFormDistMap **retmap,
					GError **error);
gboolean qtec_xform_write_dist_to_file(const gchar *fileloc,
				       QtecXFormDistMap *map, GError **error);
gboolean qtec_xform_write_dist_to_stream(GOutputStream *stream,
					 QtecXFormDistMap *map, GError **error);
gboolean qtec_xform_gain_rb_is_set(QtecXForm *xform);
gboolean qtec_xform_dist_rb_is_set(QtecXForm *xform);

G_END_DECLS
#endif
