/*
 * @Author: Dimitrios Katsaros
 * @Date:   2018-01-09
 */

#ifndef __QTEC_LIB_XFORM_DISTORTION_H__
#define __QTEC_LIB_XFORM_DISTORTION_H__

#include <qtec/xform/distortion/map.h>
#include <qtec/xform/homography.h>
#include <qtec/xform/error/map.h>
#include <qtec/cairo/cairo.h>

/* Distortion specific operations */

G_BEGIN_DECLS

typedef struct _QtecDistortion QtecDistortion;

/* core functions */
QtecDistortion *qtec_xform_dist_new(QtecXFormDistMap *map);
QtecDistortion *qtec_xform_dist_new_from_array(QtecArray *arr);
void qtec_xform_dist_free(QtecDistortion *dist);

gboolean qtec_xform_dist_prepare(QtecDistortion *dist);
gboolean qtec_xform_dist_write(QtecXForm *xform, QtecDistortion *dist);
gboolean qtec_xform_dist_check_readback(QtecXForm *xform, QtecDistortion *dist,
					gfloat minthres, gfloat maxthres,
					gfloat *difference);

QtecXFormDistMap *qtec_xform_dist_get_map(QtecDistortion *dist);
QtecArray *qtec_xform_dist_get_array_ref(QtecDistortion *dist);

/* processing functions */
gboolean qtec_xform_dist_rotate(QtecDistortion *dist, gdouble degrees);
gboolean qtec_xform_dist_rotate_center(QtecDistortion *dist, gdouble degrees,
				       QtecGeometryPoint2d center);
gboolean qtec_xform_dist_lens_correct(QtecDistortion *dist, gdouble k1,
				      gdouble k2);
gboolean qtec_xform_dist_lens_correct_cv_simple(
	QtecDistortion *dist, gdouble k1, gdouble k2, gdouble k3, gdouble p1,
	gdouble p2, QtecGeometryPoint2d center, gdouble fx, gdouble fy);
gboolean qtec_xform_dist_lens_correct_cv_full(
	QtecDistortion *dist, gdouble k1, gdouble k2, gdouble k3, gdouble k4,
	gdouble k5, gdouble k6, gdouble p1, gdouble p2,
	QtecGeometryPoint2d center, gdouble fx, gdouble fy);
gboolean qtec_xform_dist_apply_homography(QtecDistortion *dist,
					  QtecXformHomography *H,
					  gboolean invert);
gboolean qtec_xform_dist_perspective_correct(QtecDistortion *dist,
					     QtecGeometryQuad2d src_points,
					     QtecGeometryQuad2d dst_points);

/* post processing functions */
gboolean qtec_xform_dist_line_skip(QtecDistortion *dist, gfloat width_factor,
				   gfloat height_factor);
gboolean qtec_xform_dist_crop(QtecDistortion *dist, guint x, guint y, guint w,
			      guint h);
gboolean qtec_xform_dist_apply_max_valid_region(QtecDistortion *dist,
						guint error_mask);
gboolean qtec_xform_dist_gen_error_map_ppm(QtecDistortion *dist,
					   const gchar *location);

//distortion map visualization options
gboolean qtec_xform_dist_vis_draw_rects(QtecXFormDistMap *map,
					QtecCairoImage **img,
					gdouble granularity);
gboolean qtec_xform_dist_vis_draw_grid(QtecXFormDistMap *map,
				       QtecCairoImage **img,
				       gdouble granularity);
gboolean qtec_xform_dist_vis_draw_content_grid(QtecXFormDistMap *map,
					       QtecCairoImage **img,
					       gdouble granularity);
gboolean qtec_xform_dist_vis_draw_arrows(QtecXFormDistMap *map,
					 QtecCairoImage **img,
					 gdouble granularity);
gboolean qtec_xform_dist_vis_draw_polar(QtecXFormDistMap *map,
					QtecCairoImage **img,
					gdouble *max_dist);
gboolean qtec_xform_dist_vis_draw_magic_pxs(QtecXFormDistMap *map,
					    QtecCairoImage **img);

gboolean qtec_xform_dist_vis_apply_map(QtecXFormDistMap *map, QtecArray *img,
				       QtecArray **out);
gboolean qtec_xform_dist_vis_apply_map_compact(QtecXFormDistMap *map,
					       QtecArray *img, QtecArray **out,
					       gboolean compact);

G_END_DECLS
#endif
