/*
 * @Author: Dimitrios Katsaros
 * @Date:   2018-05-01
 */

#ifndef __QTEC_LIB_XFORM_ERROR_MAP_H__
#define __QTEC_LIB_XFORM_ERROR_MAP_H__

#include <qtec/core/core.h>
#include <qtec/core/array.h>
#include <qtec/xform/core.h>

/* An error map for expressing errors per pixel across different operations */

G_BEGIN_DECLS

struct _QtecXformErrorPixel {
	guint mask;
	/* a gvalue for transferring any data relevant to error type */
	GValue value;
};
typedef struct _QtecXformErrorPixel QtecXformErrorPixel;
typedef struct _QtecXformErrorMap QtecXformErrorMap;

enum {
	QTEC_XFORM_ERROR_MAP_NONE = 0,
	/* Generic error case */
	QTEC_XFORM_ERROR_MAP_ERROR = QTEC_BIT(0),
	/* Operation specific errors */
	QTEC_XFORM_ERROR_MAP_PREPAIR = QTEC_BIT(1),
	QTEC_XFORM_ERROR_MAP_LINE_SKIP = QTEC_BIT(2),
	QTEC_XFORM_ERROR_MAP_COMPARE = QTEC_BIT(3),
	QTEC_XFORM_ERROR_MAP_CROP = QTEC_BIT(4),
	QTEC_XFORM_ERROR_MAP_MAX_VALID_REG = QTEC_BIT(5),
};

/* Create new error map at maximum device format size */
QtecXformErrorMap *qtec_xform_error_map_new(QtecXForm *xf);
/* Create new error map to fit to target array */
QtecXformErrorMap *qtec_xform_error_map_new_from_array(QtecArray *array);
void qtec_xform_error_map_free(QtecXformErrorMap *map);
/* Retrieve the underlaying error map */
QtecArray *qtec_xform_error_get_array(QtecXformErrorMap *map);

/* Validation function. Checks that the array fits the error map if the error
 * map is set. Functions should call this before performing operations.
 */
QtecXformErrorMap *qtec_xform_error_map_validate(QtecXformErrorMap *map,
						 QtecArray *arr);
QtecXformErrorPixel *qtec_xform_error_map_2d_pix(QtecXformErrorMap *map,
						 guint x, guint y);
QtecXformErrorPixel *qtec_xform_error_map_pix(QtecXformErrorMap *map,
					      guint pos);
gboolean qtec_xform_error_map_gen_ppm(QtecXformErrorMap *map,
				      const gchar *fileloc, gfloat maxgradiant,
				      GError **error);

G_END_DECLS
#endif
