/*
 * @Author: Dimitrios Katsaros
 * @Date:   2018-01-09
 */

#ifndef __QTEC_LIB_XFORM_GAIN_H__
#define __QTEC_LIB_XFORM_GAIN_H__

#include <qtec/xform/gain/map.h>

/* Gain specific operations */

G_BEGIN_DECLS

typedef enum {
	QTEC_XFORM_GAIN_OFFSET_AUTO,
	QTEC_XFORM_GAIN_OFFSET_ZERO,
} QtecXformGainCalcMode;

gdouble qtec_xform_gain_calc_scale_auto_offset(guint min, guint max);
gdouble qtec_xform_gain_calc_scale_zero_offset(guint min, guint max);
gdouble qtec_xform_gain_calc_offset(gdouble scale, guint min, guint max);

/* Helper functions for validating the scale offset values with the device and
 * converting them
 */
gboolean qtec_xform_validate_scale(QtecXForm *xform, guint scaleval);
gdouble qtec_xform_convert_scale(guint scaleval);
gboolean qtec_xform_validate_offset(QtecXForm *xform, guint offsetval);
gdouble qtec_xform_convert_offset(guint offsetval);

gboolean qtec_xform_have_scale(QtecXForm *xform);
gboolean qtec_xform_set_scale(QtecXForm *xform, gdouble scaleval);
gboolean qtec_xform_set_offset(QtecXForm *xform, gdouble offsetval);

gboolean qtec_xform_gain_calculate_uniform_map(QtecXForm *xform,
					       QtecArray *inp_array,
					       gdouble *res_offset,
					       gdouble *res_scale,
					       QtecXFormGainMap **res_map,
					       QtecXformGainCalcMode mode);
gboolean qtec_xform_gain_compare(QtecXFormGainMap *gain1,
				 QtecXFormGainMap *gain2, gfloat *ret);

gboolean qtec_xform_gain_calc_vignetting_corr(QtecXForm *xform, gdouble lens_mm,
					      gdouble sensor_px_size_um,
					      gdouble *res_offset,
					      gdouble *res_scale,
					      QtecXFormGainMap **res_map,
					      QtecXformGainCalcMode mode);

G_END_DECLS
#endif
